#pragma warning( disable: 4786 )

#include "CodeGenerator.h"

#include <fstream>

using namespace std;

CodeGenerator::CodeGenerator( const char* cn, const char* d, std::vector< std::string >& fs, bool su, bool td )
{
   className = cn;
   dir = d;
   for( int i = 0; i < fs.size(); i++ )
   {
      funcs.push_back( fs[ i ] );
   }
   setUp = su;
   tearDown = td;
}

int CodeGenerator::generateCode()
{
   int r = generateHeader();
   if( r != 0 )
      return r;
   r = generateBody();
   if( r != 0 )
      return r;

   return 0;
}

// ************************* private functions **************************

int CodeGenerator::generateHeader()
{
   headerName = dir + "\\" + className + ".h";

   ofstream header( headerName.c_str() );

   header << "#ifndef __";
	header << className;
	header << "_h\n";
	header << "#define __";
	header << className;
	header << "_h\n\n";

	header << "#include <cppunit/TestCase.h>\n";
	header << "#include <cppunit/TestSuite.h>\n";
	header << "#include <cppunit/Test.h>\n";
	header << "#include <cppunit/TestCaller.h>\n\n";

	header << "#include <string>\n\n";

	header << "class ";
	header << className;
	header << " : public CppUnit::TestCase\n";
	header << "{\n";
	header << "public:\n";
	header << "\t";
	header << className;
	header << "( std::string name ) : TestCase( name ) {}\n\t";
	header << className;
	header << "() {}\n\n";

   for( int i = 0; i < funcs.size(); i++ )
   {
		header << "\tvoid ";
		header << funcs[ i ];
		header << "();\n";
   }

   if( setUp )
      header << "\tvoid setUp();\n";
   if( tearDown )
      header << "\tvoid tearDown();\n";

	header << "\tstatic CppUnit::Test *suite ();\n";
	header << "};\n\n";

	header << "#endif // __";
	header << className;
	header << "_h\n";

	header << "// automatically generated by UnitTestAddIn, v.1.0\n";

   header.close();
   return 0;
}

int CodeGenerator::generateBody()
{
   bodyName = dir + "\\" + className + ".cpp";

   ofstream body( bodyName.c_str() );

   body << "#include \"";
	body << className;
	body << ".h\"\n\n";

	body << "using namespace std;\n";
	body << "using namespace CppUnit;\n\n";

   for( int i = 0; i < funcs.size(); i++ )
   {
		body << "void ";
		body << className;
		body << "::";
		body << funcs[ i ];
		body << "()\n{\n\t// TODO - insert actual test code here\n}\n\n";
   }

   if( setUp )
   {
      body << "void " << className << "::setUp()\n";
      body << "{\n\t// TODO - add initialization code here\n}\n\n";
   }

   if( tearDown )
   {
      body << "void " << className << "::tearDown()\n";
      body << "{\n\t// TODO - add cleanup code here\n}\n\n";
   }

	body << "CppUnit::Test *";
	body << className;
	body << "::suite ()\n{\n";
	body << "\tCppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite( \"";
	body << className;
	body << "\" );\n";

   for( int j = 0; j < funcs.size(); j++ )
   {
		body << "\tsuiteOfTests->addTest (new CppUnit::TestCaller<";
		body << className;
		body << ">(\"";
		body << funcs[ j ];
		body <<  "\", &";
		body << className;
		body << "::";
		body << funcs[ j ];
		body << " ));\n";
	}


	body << "\treturn suiteOfTests;\n}\n";
	body << "// automatically generated by UnitTestAddIn, v.1.0\n";
   body.close();
   return 0;
}

